# The code accompaning `Application of pre-trained transformers to estimate the difficulty of closed questions using question and choice text`


This contains the code used to produce the results in the thesis..
It contains a number of subdirectories that are somewhat separate and somewhat interdependent.

Most of the subdirectories have their own `README.md`.

One thing that almost all directories have in common is their general [Python toolchain](#common-python-toolchain).

## Overview over the subdirectories

* [common_py](./common_py/) contains some utilities that have been used across projects.
* [raw_data](./raw_data/) contains a bunch of gitignores ignoring non-public data and READMEs explaining what data should be there. This way, other subdirectories can reference it by path.
* [enem_aggregate](./enem_aggregate/) contains the code that converts the raw data from enem, along with the info from Marinho, into a much simpler unified CSV containing the aggregated interactions.
* [moodle_extract](./moodle_extract/) contains the code that parsed the moodle question xml and bundled it together with the experimental data in the form of a CSV.
* [qde_model_code](./qde_model_code/) contains the heart of the thing, the code that trained the ml model on the data generated by the other projects.
* [data_explore](./data_explore) (may not be added) contains some (not all) of my Python scripts that were used to gain insight into the data and often to plot the data.

### Configuration

None of the projects here have a defined CLI.
Configuration is done in the source files themselves.
I did not want to add a dependency or the complexity of having a CLI (which does not suck), and frankly with how fast the underlying code and thus the API changes, it would have been a lot of work that would have just hindered experimentation.

An effort was made to have the regularly changing configuration of each potential entry point in few places, and not spread over the repository.

## Common Python toolchain

The project uses Python 3.10+, and `python-poetry` (`1.5.1`) has been used for other Python package dependencies and tools.

If your system has an older version of python and cannot be easily upgraded (Ubuntu/Debian, looking at you), you may want to look at [`pyenv`](https://github.com/pyenv/pyenv).

To install poetry, just follow [the guide](https://python-poetry.org/docs/#installation).

### Poetry

Poetry is generally similar to `npm`.

In general, unless you want to change dependencies, all you need to do is run `poetry install` in the subproject directories (they contain a `pyproject.toml` file) and then run python files with `poetry run python <whateverfile.py>`.

Poetry manages dependency versions in 2 files, one of which is dev-facing: 
`pyproject.toml` contains the versions as defined(ish) by the developer. It follows the rules of [semantic versioning] (https://semver.org/).
You can add a dependency (also to this file) by being in the project directory and running `poetry add <pypi.org name>`.

There is another file. The `poetry.lock` file should also be version controlled, but not edited by you.
It contains the actual versions of the dependencies that poetry installs.
This should solve the "but it worked on my pc" problem that plagues tools like `pip`.

Poetry installs dependencies in a [python virtual environment](https://docs.python.org/3/library/venv.html#module-venv) which it manages for you.
If you want to access this virtual environment, you can see information about it with `poetry env info`.
This can be useful with tools like `jupyter`, or to get the IDE to autocomplete.

Example output:
```sh
Virtualenv
Python:         3.11.3
Implementation: CPython
Path:           /home/<user>/.cache/pypoetry/virtualenvs/qde-model-code-9-Jx55oZ-py3.11
Executable:     /home/<user>/.cache/pypoetry/virtualenvs/qde-model-code-9-Jx55oZ-py3.11/bin/python
Valid:          True

System
Platform:   linux
OS:         posix
Python:     3.11.3
Path:       /usr
Executable: /usr/bin/python3.11
```

For example, with my Visual Studio code (vscode) that has the Python extension installed, "all I have to do" to get completions, documentation on mouse hover, and other niceties for my dependencies (numpy, tensorflow, etc) is to open the project in a vscode instance, then click on the number in the lower right corner of the editor (after "Python"), then click `+ enter interpreter path`, and then copy and paste the (first) `Executable` path from `poetry env info` into there (so in this case `/home/<user>/.cache/pypoetry/virtualenvs/qde-model-code-9-Jx55oZ-py3.11/bin/python`).

These instructions are all untested and do not include pictures, so good luck. Sadly I had to limit the resources to spend on this documentation in favour of the main work.

Don't be intimidated by all the big words being thrown around here.
You don't need to understand most of them at first, and eventually you'll develop an understanding for them.
ChatGPT can probably help you understand them, I haven't tried it for that!

### Other tools & choices for Python

My Python code contains a lot of typing.
This is mostly informal in nature (for me as well as the IDE), and often does not pass type checking.
Unfortunately, there seems to be no real standard for Python type checking anyway.

However, some of the larger projects have occasionally been checked with `mypy` (a typechecker) and ruff (a linter), and some select offenders have been fixed.

What has not been intentionally fixed is a lot of errors about values potentially being None (which is also offered in a somewhat unhelpful way by mypy, btw).
Python does not have any syntactic sugar to make these easier to handle (such as [optional chaining](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Operators/Optional_chaining) or chaining non-null assertions).

Also usually the "wrong" code will lead to the same results a non-null assertion would lead to in other languages, an exception saying "that thing is None!.
Sadly I'm not aware of a way to tell mypy that this is fine, so I'll ignore these kinds of errors.

### Why a Monorepo, why separate projects

This was created as a monorepo from the individual projects.
These were usually already (local) git repos.
Having them separate was a good thing, I had no global structure and a lot of copying and pasting in the beginning while figuring out the structure.
And the project history may also contain data that was not intended to be shared.
Doing tasks with a lot of data wrangling doesn't easily translate to the things that are established in software repositories.

I considered implementing these as git submodules, but this was not done mainly because of the complexity.

Git subrepositories don't come without gotchas, the [official documentation](https://git-scm.com/book/en/v2/Git-Tools-Submodules) has an entire subsection about these, and theres some others that may be more problematic.

This project will already be complex to run.
I do not need to make them learn how to handle subrepositories as well.

Adding 
* the aforementioned potential data leakage, which is labor-intensive to fix
* While this probably already contains a lot of not-so-glamorous code, other stuff is in the past, and I'd prefer it not to be available to the worldwide web.
* Submodules make multiple remote repositories difficult from my understanding (as they point to the submodules per remote url), and I am considering cloning this into a private repo in case my HFT gitlab repo gets deleted.
