"A few unrelated tools that may be reused across the different projects"

from typing import NamedTuple
from numpy import typing as npt
import numpy as np

def UNREACHABLE(reason: str = "Not specified"):
    "Used to assert that the current branch of execution is unreachable under intended circumstances."
    raise Exception(f"Should be unreachable because: {reason}")

import warnings
import functools

def identity(arg):
    "just returns the argument."
    return arg

class Statistics(NamedTuple):
    min: float
    max: float
    mean: float
    std_dev: float

    def from_numpy(source: npt.ArrayLike):
        return Statistics(
            np.min(source),
            np.max(source),
            np.mean(source),
            np.std(source),
        )

def deprecated(reason: str = "No reason given"):
    """
    This is a decorator which can be used to mark functions
    as deprecated. It will result in a warning being emitted
    when the function is used.

    Because I'm to lazy to look up how to do it right, you'll always need to call this decorator:

    ```py
    # correct
    @deprecated()
    def some_function():
        pass

    # not correct
    @deprecated
    def some_other_function():
        pass
    ```

    You can optionally provide a reason in that call (that is the optional parameter of this function)
    """
    def inner_deprecated(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            warnings.simplefilter('always', DeprecationWarning)  # turn off filter
            warnings.warn(
                f"Call to deprecated function {func.__name__}.\nReason for deprecation: {reason}",
                category=DeprecationWarning,
                stacklevel=2,
            )
            warnings.simplefilter('default', DeprecationWarning)  # reset filter
            return func(*args, **kwargs)
        return wrapper
    return inner_deprecated
