from common_py.plotting import *
import polars as pl
import os
import numpy as np
import json

if __name__ == "__main__":
    df = pl.scan_csv("../enem_aggregate/GENERATED/localGen_result.csv").collect()
    # print(df.filter(pl.col("irt_b") > 10))
    irt_marinho = df["irt_b_marinho"]
    correctness = df["correctness"]
    filter_for_nulls = (irt_marinho.is_not_null() & correctness.is_not_null())
    df = df.filter(filter_for_nulls)
    irt_marinho = df["irt_b_marinho"]
    correctness = df["correctness"]

    filename = os.path.basename(__file__).split('.')[0]

    plot = PlotData(
        title="IRT difficulty (Marinho) vs correctness",
        save_path=f"./{filename}.pdf",
        x_ax=AxData("Marinho IRT Difficulty"),
        y_ax=AxData("Correctness (all participants)"),
        graphs=[
            Plot(xData=df["irt_b_marinho"].to_list(), yData=df["correctness"].to_list(), kwargs={"marker": "x", "linestyle": "None"}),
        ]
    ).create(add_legend=False)
    plot.draw_and_save()
    linear_corrcoef = np.corrcoef(irt_marinho.to_numpy(), correctness.to_numpy())[0,1]

    with open(f"./{filename}.json", mode="w") as fh:
        json.dump({
            "correlations_coefficient": linear_corrcoef,
        }, fh, indent="\t")
    for topic in ["MT", "LC", "CH", "CN"]:
        df_topic = df.lazy().filter(pl.col("topic") == topic).collect()

        irt_marinho = df_topic["irt_b_marinho"]
        correctness = df_topic["correctness"]
        # print(df.filter(pl.col("irt_b") > 10))
        plot = PlotData(
            title="IRT difficulty (Marinho) vs correctness",
            save_path=f"./{filename}_{topic}.pdf",
            x_ax=AxData("Marinho IRT Difficulty"),
            y_ax=AxData("Correctness (all participants)"),
            graphs=[
                Plot(xData=irt_marinho.to_list(), yData=correctness.to_list(), kwargs={"marker": "x", "linestyle": "None"}),
            ]
        ).create(add_legend=False)
        plot.draw_and_save()