from common_py.plotting import *
import polars as pl
import numpy as np
import os
import json

if __name__ == "__main__":
    df = pl.scan_csv("../dataextraction/enem_aggregate/GENERATED/localGen_result.csv").collect()
    # df = df.filter(pl.col("qname").str.contains("2017").is_not())
    print(df.filter(pl.col("irt_b") > 10))
    print(df.select(pl.col("^.*irt.*$")).describe())
    inep_irt = df["irt_b"]
    marinho_irt = df["irt_b_marinho"]
    filter_for_nulls = (inep_irt.is_not_null() & marinho_irt.is_not_null())
    inep_irt: pl.Series = inep_irt.filter(filter_for_nulls)
    marinho_irt: pl.Series = marinho_irt.filter(filter_for_nulls)

    # fitting and correlation REALLY dont like the outliers in INEP data,
    # so we limit the range using quantiles.
    # I got here thinking of confidence intervals,
    # however this does not assume a normal distribution,
    # unlike most default models of confidence intervals
    border_exclude = 0.01
    "has to be in [0,0.5)"

    correlation_range = [inep_irt.quantile(border_exclude), inep_irt.quantile(1 - border_exclude)]
    filter_for_range = inep_irt.is_between(*correlation_range)
    inep_irt_rangefiltered: pl.Series = inep_irt.filter(filter_for_range)
    marinho_irt_rangefiltered: pl.Series = marinho_irt.filter(filter_for_range)

    linear_corrcoef = np.corrcoef(inep_irt_rangefiltered, marinho_irt_rangefiltered)[0,1]
    # see https://numpy.org/doc/stable/reference/routines.polynomials.classes#fitting
    [b, m] = np.polynomial.Polynomial.fit(inep_irt_rangefiltered.to_numpy(), marinho_irt_rangefiltered.to_numpy(), deg = 1).convert()
    filename = os.path.basename(__file__).split('.')[0]
    plot = PlotData(
        title="IRT difficulty by Marinho vs INEP",
        save_path=f"./{filename}.pdf",
        x_ax=AxData("INEP", bottom=inep_irt.quantile(0.001), top=inep_irt.quantile(0.999)),
        y_ax=AxData("Marinho"),
        graphs=[
            Plot(xData=inep_irt.to_list(), yData=marinho_irt.to_list(), kwargs={"marker": "x", "linestyle": "None"}),
            Function(function=lambda x: m*x + b, range_from=[correlation_range], label=f"regression over correlation range"),
        ]
    ).create().draw_and_save()
    with open(f"./{filename}.json", mode="w") as fh:
        json.dump({
            "correlations_coefficient": linear_corrcoef,
            "filter_range": correlation_range,
        }, fh, indent="\t")