
use jemallocator::Jemalloc;
// just followiing polars suggestions for speed, this is an untested presumed speedup.
#[global_allocator]
static GLOBAL: Jemalloc = Jemalloc;

use polars::{lazy::{dsl, frame::OptState}, prelude::{self as pl, LazyFileListReader as _, SerWriter as _, *}, functions::diag_concat_df};
use enem_aggregate::ENEM_QUESTION_ANSWER_OPT;

fn main() {
    // DEV:
    let root_dir = "./official_data";
    // let root_dir = "../../../raw_data/brasilian/official_data";

    let question_ident = [
        "#question", 
        "topic",
        "ordering_id",
        "year",
    ].map(|elem| dsl::col(elem));

    let mut interactions_eval = Vec::<pl::DataFrame>::new();

    // DEV:
    for year in 2009..=2021 {
        dbg!(year);
        let interactions = pl::LazyCsvReader
            ::new(format!("{root_dir}/{year}/DADOS/MICRODADOS_ENEM_*.csv"))
            .with_delimiter(b';')
            .with_encoding(pl::CsvEncoding::LossyUtf8)
            .finish()
            .unwrap();

        let topics = [
            "CN",
            "CH",
            "LC",
            "MT",
        ];

        let interactions = interactions.clone()
            .with_optimizations(OptState{
                common_subplan_elimination: false,
                // doesnt actually work right now
                streaming: true,
                predicate_pushdown: true,
                projection_pushdown: true,
                slice_pushdown: true,
                type_coercion: true,
                // making this false causes a panic!
                file_caching: true,
                simplify_expr: true,
            })
            .select(
                topics.into_iter()
                    .map(|topic| {
                        dsl::as_struct(&[
                            dsl::col(&format!("CO_PROVA_{topic}"))
                                .alias(&format!("ordering_id")),
                            dsl::col(&format!("TX_RESPOSTAS_{topic}"))
                                .alias(&format!("answers")),
                        ]).alias(topic)
                    })
                    .chain([dsl::col("NU_INSCRICAO").alias("#student")])
                    .collect::<Vec<_>>()
            )
            .melt(pl::MeltArgs {
                id_vars: vec!["#student".into()],
                value_vars: topics.map(Into::into).to_vec(),
                streamable: true,
                variable_name: Some("topic".into()),
                value_name: Some("interactions".into()),
            })
            .filter(dsl::col("interactions").struct_().field_by_name("ordering_id").is_null().not())
            .unnest(["interactions"])
            .explode(&[
                dsl::col("answers"),
            ])
            .groupby(&[
                dsl::col("#student"), 
                dsl::col("topic")
            ])
            .agg(&[
                dsl::col("ordering_id").unique().first(),
                dsl::col("answers"),
                dsl::lit(year).alias("year"),
                dsl::col("answers").cumcount(false).cast(pl::DataType::UInt8).alias("#question"),
            ])
            .explode([
                "answers",
                "#question",
            ])
        ;

        // dbg!(&question_ident);

        let interactions = interactions
            .sort_by_exprs(&question_ident, [false, false, false], true)
        ;

        // dbg!(interactions.clone().fetch(5).unwrap());

        let interactions_coll = interactions.clone()
            .groupby(&question_ident)
            .agg([
                dsl::col("answers").value_counts(false, false),
            ])
            // .sort_by_exprs(&join_cols, [false, false, false], true)
        ;

        // dbg!(interactions_coll.clone()
        //     .fetch(5).unwrap())
        // ;

        interactions_eval.push(
            interactions_coll.clone()
                .with_columns(
                    ENEM_QUESTION_ANSWER_OPT.map(|row_title| {
                        dsl::col("answers").map(move |col| {
                            let accum_counts = col.list().unwrap().into_iter().map(|elements| {
                                let elements = elements.unwrap();
                                let elements = elements.struct_().unwrap();
                                let [choice, count] = elements.fields() else {unreachable!()};
                                let accum_choice = choice.utf8().unwrap().equal(row_title);
                                let count: i64 = count.u32().unwrap().filter(&accum_choice).unwrap().cast(&pl::DataType::Int64).unwrap().sum().unwrap();
                                count
                            });
                            Ok(Some(Series::from_iter(accum_counts)))
                        }, Default::default()).alias(&format!("#{row_title}_answers"))
                    })
                )
                .with_column(
                    dsl::col("answers").map(move |col| {
                        let accum_counts = col.list().unwrap().into_iter().map(|elements| {
                            let elements = elements.unwrap();
                            let elements = elements.struct_().unwrap();
                            let [_, count] = elements.fields() else {unreachable!()};
                            let count: i64 = count.u32().unwrap().cast(&pl::DataType::Int64).unwrap().sum().unwrap();
                            count
                        });
                        Ok(Some(Series::from_iter(accum_counts)))
                    }, Default::default()).alias("#total_answers")
                )
                .select(&[
                    "#question",
                    "topic",
                    "ordering_id",
                    "year",
                    // "correct_perc",
                    "#total_answers",
                    "^#._answers$",
                ].map(dsl::col))
                .sort_by_exprs(&question_ident, [false, false, false], true)
                // fetch here seems somewhat problematic, causes nulls which dissappear with a longer fetch
                // DEV:
                // .fetch(500).unwrap()
                // also despite trying to only trigger collect() where it should not take up much memory, 
                // this does indeed end up taking faaar too much RAM (debugging with explain or to_dot above!)
                .collect().unwrap()
        );

        // std::io::BufWriter::new(std::fs::File::create("polars.unopt.dot").unwrap()).write_all(
        //     interactions_eval.last().unwrap().clone()
        //         .sort_by_exprs(&join_cols, [false, false, false], true)
        //         .to_dot(false).unwrap().as_bytes()
        // ).unwrap();

        // println!("{}",
        //     interactions_eval.last().unwrap().clone()
        //         .with_streaming(true)
        //         .explain(true).unwrap()
        // );

        dbg!(interactions_eval.last().unwrap().clone());
    }

    let mut interactions_eval = diag_concat_df(&interactions_eval).unwrap();

    dbg!(interactions_eval.clone());

    pl::CsvWriter::new(std::fs::File::create("./clusterGen_interactions_agg.csv").unwrap()).finish(&mut interactions_eval).unwrap();
}
