"""
This should probably be your main entrypoint, as this, when choosen as entry point,
will go through through the raw data provided and generate `./GENERATED/result.csv`,
which contains the questions alongside all their raw data from moodle and the csv file.
"""
from moodle_to_generic_questions import GenericQuestion
from map_generic_question_to_str import *
import polars as pl
import fuzzywuzzy.process as fuzzyProc

def combineQuizXmlAndExperimentalData(
    questionsDataList: list[GenericQuestion],
    experimentalData: pl.LazyFrame,
):
    questionsData: pl.LazyFrame = pl.DataFrame(questionsDataList).lazy()
    questionsData2: pl.LazyFrame = questionsData.with_columns([
        # Could be made more efficient with .select call before collect
        pl.col("name").apply(lambda item: fuzzyProc.extractOne(item, experimentalData.collect()["title"])[0], return_dtype=pl.datatypes.Utf8).alias("title"),
        pl.Series([len(genericQuestion.answerOptions) for genericQuestion in questionsDataList]).alias("answer_count"),
        pl.Series([mapGenericQuestionToStr(genericQuestion) for genericQuestion in questionsDataList]).alias("question_and_answer_text"),
        pl.Series([mapGenericQuestionToQuestionStr(genericQuestion) for genericQuestion in questionsDataList]).alias("question_text"),
        pl.Series([mapGenericQuestionToAnswerStr(genericQuestion) for genericQuestion in questionsDataList]).alias("answer_text"),
    ])
    questionsData3: pl.LazyFrame = questionsData2.join(experimentalData, on="title")
    # questionsData4: pl.LazyFrame = questionsData3.with_columns([
    #     pl.struct(["name", "title"]).apply(lambda row: fuzz.ratio(row["name"], row["title"])).alias("similarity"),
    # ])
    # questionsData5: pl.LazyFrame = questionsData4.sort(pl.col("similarity"), descending=False)
    # questionsData6: pl.LazyFrame = questionsData3.select(["name", "questionText", "answerOptions", "mean_share_total_points"])
    return (questionsData3
        .select([
            "lecture", 
            "retries_allowed", 
            "#quiz_participants", 
            "mean_share_total_points",
            "questionType",
            "answer_count",
            "^.*_text$", 
        ])
    )

if __name__ == "__main__":
    from extract_moodle_xml import extractQuestionsFromMoodleQuizXml
    from moodle_to_generic_questions import mapToGenericQuestionFormat
    df = combineQuizXmlAndExperimentalData(
        mapToGenericQuestionFormat(extractQuestionsFromMoodleQuizXml(
            map(lambda subject: (subject, f"../raw_data/hft/{subject}/quiz.moodle.xml"), ["ASV", "KI", "MLDM", "PGM_1_2"])
        )),
        pl.read_csv("../raw_data/hft/experimental_data_extract.csv").lazy(),
    ).collect()
    print(df)
    df.write_csv("./GENERATED/result.csv")
    # df.write_csv("../qde_model_code/sources/hft_prepared.csv")
    
    # print(df.filter((pl.col("name").str.contains(r"Regression.*"))).head().collect())
