import numpy as np
import polars as pl
from typing import NamedTuple
from numpy import typing as npt
from common_py.plotting import *
from common_py.utils import Statistics
import re
import os
import json

if __name__ == '__main__':
    analyzedDir = "./results/clean/"

    varied_variable = "learning_rate"
    varied_variable_pretty = "Learning rate for the combined model"

    train_correlation_coefficient: list[float] = []
    valid_correlation_coefficient: list[float] = []
    train_rmse: list[float] = []
    varied_variable_vals = []

    for modelConfigIdentifier in [
        match.group(1) 
        for match
        in [
            re.match(r"(hash\[[\d\w-]*\])_config.json", fileName)
            for fileName
            in os.listdir(analyzedDir)
        ]
        if match
    ]:
        basePath = f"{analyzedDir}/{modelConfigIdentifier}"
        with open(f"{basePath}_config.json") as fp:
            config = json.load(fp)
        history = pl.read_csv(f"{basePath}_history.csv")
        train_loss_sorted = np.sort(history['loss'])
        val_loss_sorted = np.sort(history['val_loss'])

        target = config["target"]

        varied_variable_vals.append(config["training"][varied_variable])

        target_sqErr_col = f"{target}_err_squared"
        target_absErr_col = f"{target}_err_abs"

        predictions: pl.DataFrame = (pl.read_csv(f"{basePath}_predictions.csv", dtypes={"split": pl.Categorical})
            .with_columns(
            (pl.col(target) - pl.col(f"predicted_{target}")).abs()
                .alias(target_absErr_col),
            (pl.col(target) - pl.col(f"predicted_{target}")).pow(2)
                .alias(target_sqErr_col)
            )
        )
        predictions_valid: pl.DataFrame = (predictions
            .filter(pl.col("split") == "validate")
        )
        predictions_train: pl.DataFrame = (predictions
            .filter(pl.col("split") == "train")
        )
        stat_sqErr_train = Statistics.from_numpy(predictions_train[target_sqErr_col].to_numpy())
        stat_sqErr_valid = Statistics.from_numpy(predictions_train[target_sqErr_col].to_numpy())
        train_rmse.append(
            np.sqrt(stat_sqErr_train.mean)
        )
        train_correlation_coefficient.append(
            np.corrcoef(predictions_train[target].to_numpy(), predictions_train[f"predicted_{target}"].to_numpy())[0,1]
        )
        valid_correlation_coefficient.append(
            np.corrcoef(predictions_valid[target].to_numpy(), predictions_valid[f"predicted_{target}"].to_numpy())[0,1]
        )

    PlotData(
        [
            Plot(train_correlation_coefficient, varied_variable_vals, kind="scatter", label="training data"),
            Plot(valid_correlation_coefficient, varied_variable_vals, kind="scatter", label="validation data"),
        ],
        x_ax=AxData(varied_variable_pretty),
        y_ax=AxData("Linear correlation coefficient"),
        save_path=f"{analyzedDir}/correlation_variance.pdf",
    ).create().draw_and_save()