#!/bin/sh

set -o errexit

AUTHENTICATED_SSH_HOST="uniClusterIp"
# Authenticated ssh Host.
# eg. set (this also ought to work on windows with git bash + per default preinstalled openssh):
# -------------------
# # ~/.ssh/config
# Host uniCluster
#         HostName bwunicluster.scc.kit.edu
#         User <YOUR_LOGIN>
# 
# Host *
#         # reuse existing ssh connections (see man ssh_config)
#         # creates a file according to ControlPath for every (now shared) session
#         ControlMaster auto
#         ControlPath ~/.ssh/ssh_mux_%h_%p_%r
# ----------------
# and then login once, and keep that connection while executing this.
EXEC_DIR="enem/eval_paramv3" #(path relative to home)
# EXEC_DIR="hft_reset_model/lotsof_epochs_mcq_no_retries" #(path relative to home)

# print each (following) command as it gets executed (except commands from an ssh script, didnt find a way to do that at all)
set -o xtrace

poetry run python create_server_files.py

# (EXEC_DIR gets expanded on the client - the computer that executes this - on purpose)
ssh ${AUTHENTICATED_SSH_HOST} "mkdir -p ${EXEC_DIR}"

scp -r sources ./*.py train_strategy slurm.sh ${AUTHENTICATED_SSH_HOST}:${EXEC_DIR}/

# The cluster doesnt have poetry, but requirements.txt generated from poetry should suffice
# poetry also generates an venv by default, which we dont want in all circumstances on the server
# (eg when we already use a container, I've seen venv interfering with GPU support)
poetry export --only=main > requirements.txt
scp requirements.txt ${AUTHENTICATED_SSH_HOST}:${EXEC_DIR}/
rm requirements.txt

ssh ${AUTHENTICATED_SSH_HOST} <<EOF_SSH
set -o errexit

cd ${EXEC_DIR}
sbatch slurm.sh
exit
EOF_SSH