"""
These configs serve the purpose of being easily serializable into a json file for record keeping,
as well as being hashable to be able to easily correlate generated files with the config used to generate it
"""

from dataclasses import dataclass
import dataclasses
from typing import Literal

# frozen=True enables a structural hash implementation
@dataclass(frozen=True)
class TrainingConfig:
    batch_size: int = 32
    epochs: int = 25
    # earlystop_patience: int = 5
    earlystop_patience: int = 10
    learning_rate: float = 1e-4
    dropout_pretrained: float = 0
    dropout_regression: float = 0.1
    freeze_base: bool = False
    training_kind: Literal["train_validate"] | Literal["leave_one_out"] = "train_validate"
    "this does nothing atm"

@dataclass(frozen=True)
class ModelConfig:
    huggingface_name: str
    context_length: int = 512

# @dataclass(frozen=True)
# class OtherConfig:
#     seed: int = 1
#     """
#     This doesnt get applied the dataset split on purpose.
#     If the seed also influences the split, then we may select for the split instead of the actual model
#     """

@dataclass(frozen=True)
class PresetConfig:
    model: ModelConfig
    target: str = "correctness"
    # default values will usually be assigned ONCE per dataclass,
    # which would mean that all classes share the same training instance.
    # Because of this issue we use dataclasses.field
    training: TrainingConfig = dataclasses.field(default_factory=TrainingConfig)

@dataclass(frozen=True)
class AllConfig:
    model: ModelConfig
    training: TrainingConfig = dataclasses.field(default_factory=TrainingConfig)
    target: str = "correctness"
    # meta: OtherConfig  = dataclasses.field(default_factory=OtherConfig)
    seed: int = 1
    """
    This doesnt get applied the dataset split on purpose.
    If the seed also influences the split, 
    then we may select for the split instead of the model config,
    meaning we taint the split-off test data
    """
