import polars as pl
from functools import reduce
import re
from common_py.utils import deprecated
from common_py.string_processing import cleanHtml, combineStrIter, stripLines

@deprecated("""
Keeping this for posterity.
Matching URLs in the Marinho ENEM texts via regex is not an easy task,
think twice if you want to archive it.

The much easier approach that gets you most of the way is to either manually remove them all,
or apply undermatching regular excaptions manually piece by piece.

Do ensure that these are actually undermatching,
and dont remove more text in a few border cases, this was often the case for me.
""")
def removeUrls(input: str):
    # the regex below overmatches some lines
    # r"Disponivel em : .+? \d\d\d\d (\( adaptado \))?"
    # ENEM data from Marinho contains URLs, but they are also separated with whitespace, which means regular Regexes for URLs dont work.
    # also, trying to expand regular regex to match this will end up overmatching
    # this us damned hopeless. I always find further things that make it hard and dont get removed or require overmatching some required text. Might as well give up.
    return re.compile(r"https? :? ?/ / ([\w\d]+ \. )+(net|br|com|uk)", re.MULTILINE).sub("url", input)

def removeEnemSignet(input: str):
    return re.compile(r"\( ENEM (- 2021 )?\)").sub("", input)

def cleanText(input: str, doCleanHtml: bool = False):
    lineStripped = removeEnemSignet(stripLines(input))
    return lineStripped if not doCleanHtml else cleanHtml(lineStripped)

def isEnemAnswerLine(arg: str):
    return arg.startswith(" #F") or arg.startswith(" #T")

def onlyQuestionTextEnem(input: str):
    return combineStrIter(filter(lambda arg: not isEnemAnswerLine(arg), input.splitlines(keepends=True)))

def onlyAnswerText(input: str, doCleanHtml: bool = False):
    return combineStrIter(filter(isEnemAnswerLine, input.splitlines(keepends=True)))

def loadEnem(input: pl.DataFrame, num: int | None = None):
    difficulty_col = "irt_b_marinho"
    max_irt = input[difficulty_col].max()
    return (pl
        .DataFrame([
            input["question_id"],
            input["topic"],
            input["correctness"],
            input["anulled"],
            input["qname"],
            input[difficulty_col].apply(lambda f: f / max_irt).alias("irt_b_marinho_maxnorm") ,
            input["question_and_answer_text"].apply(cleanText),
            input["question_and_answer_text"].apply(onlyQuestionTextEnem).apply(cleanText).alias("question_text"),
            input["question_and_answer_text"].apply(onlyAnswerText).apply(cleanText).alias("answer_text"),
        ])
        .filter(pl.col("topic") != "LC")
        # .filter(pl.col("topic") == "MT")
        .filter(pl.col("anulled") == 0)
        .with_columns(
            # just a little explainer in the file...
            pl.lit(None).alias(f"irt_b_marinho_maxnorm={difficulty_col}/{max_irt}"),
        )
        .unique(subset="question_and_answer_text", keep='any')
        # .sample(num, seed=123)
        .sort(pl.col("qname"))
    )

def loadHft(input: pl.DataFrame):
    return (input
        # if you want to update the way answers are formatted, you MUST do so in `moodle_export`, and execute the main entry point afterward
        .filter(
            (pl.col("questionType").is_in(pl.Series([
                # "singlechoice", 
                "multichoice", 
                # "truefalse",
                # "matching", # these and the others below do not have answers if you didnt change the logic!
                # TODO: actually add these questiontypes, ATM theyre straight up ignored before we get to this repo.
                # "gapselect",
                # "ddwtos",
            ]))) 
            & (pl.col("retries_allowed") == False)
        )
    )

if __name__ == "__main__":
    hft_dataset = loadHft(
        pl.read_csv("../moodle_extract/GENERATED/result.csv")
    )

    loadEnem(
        pl.read_csv("../enem_aggregate/GENERATED/localGen_result.csv"), 
        # num=len(hft_dataset),
    ).write_csv(
        "./sources/enem_prepared.csv"
    )

    hft_dataset.write_csv(
        "./sources/hft_prepared.csv"
    )

    # loadEnem(
    #     ingestData("../raw_data/brasilian/with_corresponding/enem2.csv")
    # ).write_csv(
    #     "./sources/enem_text_difficulty.csv"
    # )