import numpy as np
import polars as pl
from typing import NamedTuple
from numpy import typing as npt
from common_py.plotting import *
from common_py.utils import Statistics, identity
import re
import os
import json
from utils import lin_correlation

if __name__ == '__main__':
    # analyzedDir = "./results/hft_reset_model/lotsof_epochsv3"
    analyzedDir = "./results/hft_reset_model/default_epochsv2"

    # per_category_train = PlotData(
    #     title='Training progress, comparison for training data',
    #     save_path=f"{basepath}/progress_comparison_train.pdf",
    #     x_ax=AxData('Epoch'),
    #     y_ax=AxData(f'Loss: MSE, logarithmic', scale="log"),
    #     # we want to add that data piece by piece, 
    #     # however with a default value for plot, 
    #     # the different PlotData instances would share a single list.
    #     graphs=[],
    # )
    # per_category_validate = PlotData(
    #     title='Training progress, comparison for validation data',
    #     save_path=f"{basepath}/progress_comparison_validate.pdf",
    #     x_ax=AxData('Epoch'),
    #     y_ax=AxData(f'Loss: MSE, logarithmic', scale="log"),
    #     graphs=[], 
    # )
    for modelConfigIdentifier in [
        match.group(1) 
        for match
        in [
            re.match(r"(hash\[[\d\w-]*\])_config.json", fileName)
            for fileName
            in os.listdir(analyzedDir)
        ]
        if match
    ]:  
        baseStr = f"{analyzedDir}/{modelConfigIdentifier}"
        with open(f"{baseStr}_config.json") as fp:
            config = json.load(fp)
        model = config["model"]["huggingface_name"]
        history = pl.read_csv(f"{baseStr}_history.csv")
        target = config["target"]
        left_outs = history["left_out"].unique().to_list()
        PlotData(
            graphs=[
                *[Plot(yData=history.filter(pl.col("left_out") == left_out)['loss'], label= f"loss for training without {left_out}") for left_out in left_outs],
            ],
            x_ax=AxData('Epoch'),
            y_ax=AxData('Loss: MSE, logarithmic axis', scale="log"),
            save_path=f"{baseStr}_history.pdf",
        ).create(add_legend=False).draw_and_save()
        # per_category_train.graphs.append(Plot(yData=train_loss, label=model))
        # per_category_validate.graphs.append(Plot(yData=val_loss, label=model))

        predictions: pl.DataFrame = (pl.read_csv(f"{baseStr}_predictions.csv", dtypes={"split": pl.Categorical})
            .with_columns(
            (pl.col(target) - pl.col(f"predicted_{target}"))
                .abs().alias(f"{target}_err_abs"))
        )
        pred_descr: pl.DataFrame = predictions.describe(percentiles=[0.1, 0.01, 0.001])
        print(pred_descr)
        stat = Statistics.from_numpy(predictions[target].to_numpy())

        PlotData(
            x_ax=AxData(label='target'),
            y_ax=AxData(label='predicted'),
            save_path=f"{baseStr}_pred.pdf",
            graphs=[
                Function(identity, range_from=predictions[target].to_numpy(), label="target"),
                Plot(yData=predictions[f"predicted_{target}"].to_list(), xData=predictions[target].to_list(), label="predicted", kwargs={"marker": "x", "linestyle": "None"}),
            ]
        ).create().draw_and_save()

        print(
            baseStr,
            "\n",
            lin_correlation(predictions[f"predicted_{target}"].to_numpy(), predictions[target].to_numpy())
        )

    # per_category_train.create().draw_and_save()
    # per_category_validate.create().draw_and_save()