#!/bin/sh
#SBATCH --partition=gpu_4_a100
#SBATCH --mail-type=ALL
#SBATCH --time=05:00:00
#SBATCH --mail-user=02masa1bif@hft-stuttgart.de
#SBATCH --output=%x_%A.log
#SBATCH --cpus-per-task=2
#SBATCH --mem=128gb
#SBATCH --gres=gpu:1

set -o errexit
# keeping some settings here for copy paste, 
# because theyre after an actual command they dont get detected by slurm
#SBATCH --time=0:45:00
#SBATCH --time=1:30:00
#SBATCH --partition=gpu_4_a100
#SBATCH --partition=dev_gpu_4_a100
#SBATCH --partition=dev_gpu_4

set -o xtrace

# execute on containers to be mostlyish repeatable

# Ensure dependencies are installed.
# Beware: containers persist, so if you mess up the environment here (eg messing up tensorflow libraries),
# you'll have to recreate a fresh container to reset
enroot start --rw -m "${PWD}:/workspace" nv_tensor_container bash <<EOF_CNT
set -o errexit
set -o xtrace
pip install --requirement requirements.txt
# this shows in the log whether the GPU was detected or not. 
# The container will regularly log that compatibility mode is unavailable,
# however that does not mean that GPU support isn't there, and generally is... weird.
# In my experience (as first pointed out by cluster support staff in a ticket)
# the compatibility mode will be logged as unavailable 
# after the first use of the container in read-write mode,
# however it still seems to work. 
# My guess is that Nvidia screwed something up in their logic that leads to this erraneous log,
# not expecting that a container is reused in that way.
# It is nvidias enroot tool itself that suggests this kind of reuse with its CLI API, 
# in addition to the gigantic size of the container files.
# Also, regarding the need for a compatibility mode,
# Nvidia seems to put drivers that are not released as stable in their containers,
# which is something that I found NOWHERE on their release page, and also contributed to my confusion.
# Because if that, the cluster, despite having the latest nvidia driver, 
# needs compatibility mode with these containers.
python<<EOF_TEST
import tensorflow as tf
print(tf.config.list_physical_devices())
EOF_TEST
EOF_CNT

# execute. Requires write privileges to autodownload models
enroot start --rw -m "${PWD}:/workspace" nv_tensor_container python cluster_entry.py


# This is kept around, in case that the container workload doesnt work for someone.
# EG. because they dont have a nvidia GPU.
# Since container workflow doesnt work yet, execute with a prepared venv instead
# module load jupyter/tensorflow/2023-03-24
# source ../venv/bin/activate

# python main.py
